from io import StringIO
import re

import pytest

from json_stream_rs_tokenizer import load


def test_free_charater():
    with pytest.raises(
        ValueError,
        match=re.escape("Invalid JSON character: 'a' at index 0"),
    ):
        list(load(StringIO("a")))


def test_letter_in_number():
    with pytest.raises(
        ValueError,
        match=re.escape(
            "A number must contain only digits.  Got 'a' at index 4"
        ),
    ):
        list(load(StringIO("[123a]")))


def test_invalid_number_starting_with_zero():
    with pytest.raises(
        ValueError,
        match=re.escape(
            "A 0 must be followed by a '.' | a 'e'.  Got '1' at index 1"
        ),
    ):
        list(load(StringIO(r"01")))


def test_invalid_character_code():
    with pytest.raises(
        ValueError,
        match=re.escape(
            "Unterminated unicode literal at end of file at index 5"
        ),
    ):
        list(load(StringIO(r'"\uz"')))



from io import BytesIO, StringIO

import pytest


@pytest.fixture(params=["str", "bytes", "str-unseekable", "bytes-unseekable"])
def to_bytes_or_str_buf(request):
    if request.param == "str":
        return lambda s: StringIO(s)
    elif request.param == "str-unseekable":

        def make_unseekable_stringio(s: str):
            sio = StringIO(s)
            sio.seekable = lambda: False
            return sio

        return make_unseekable_stringio
    elif request.param == "bytes":
        return lambda s: BytesIO(s.encode("utf-8"))
    elif request.param == "bytes-unseekable":

        def make_unseekable_bytesio(s: str):
            bio = BytesIO(s.encode("utf-8"))
            bio.seekable = lambda: False
            return bio

        return make_unseekable_bytesio
    else:
        assert False



from io import StringIO

import pytest

from json_stream_rs_tokenizer import load


@pytest.mark.parametrize(
    "s,expected",
    [
        (r"\uD83C\uDFD4\uFE0F", "\U0001f3d4\uFE0F"),
        (r"a\uD83C\uDFD4\uFE0F", "a\U0001f3d4\uFE0F"),
        (r"\uD83C\uDFD4\uFE0Fa", "\U0001f3d4\uFE0Fa"),
    ],
)
def test_unicode_surrogate_pairs(s, expected):
    assert list(load(StringIO(f'["{s}"]'))) == [expected]


@pytest.mark.parametrize(
    "s",
    [
        r"\uD83Ca",
        r"\uD83C",
        r"\uD83C\n",
        r"\uD83C\u00e4",
    ],
)
def test_invalid_unicode_surrogate_pairs(s):
    with pytest.raises(ValueError, match=".*surrogate.*"):
        assert list(load(StringIO(f'["{s}"]')))
